<?php
/**
 * Created by PhpStorm.
 * User: phoenix
 * Date: 01/04/16
 * Time: 18:32
 *
 */

/**
 * Fonction qui calcule les différentiels de scores obtenus après une partie d'un jeu à deux joueurs
 * où l'on peut perdre (0) ou gagner (1).
 * @param int $SA : score actuel du premier joueur
 * @param int $RA : score (valant 0 ou 1) du premier joueur
 * @param int $SB : score actuel du second joueur
 * @param int $RB : score (valant 0 ou 1) du second joueur
 * @param int $K : la quantité max de points gagnable ou perdable sur une partie
 * @return array : le différentiel de score du premier joueur et du second joueur
 */
srand();

function ELO_duel($SA, $RA, $SB, $RB, $K)
{
    $EA = 1/(1+10**(($SB-$SA)/400));
    $EB = 1/(1+10**(($SA-$SB)/400));
    $newRA = $K * ($RA-$EA);
    $newRB = $K * ($RB-$EB);

    if ($newRA>0)
        $newRA = ceil($newRA);
    else
        $newRA = floor($newRA);

    if ($newRB>0)
        $newRB = ceil($newRB);
    else
        $newRB = floor($newRB);

    return array($newRA, $newRB);
}

/**
 * Fonction qui calcule les différentiels de scores obtenus après une partie à count($S) joueurs
 * en considérant que $R correspond au classement des joueurs dans ce jeu (du type array(1,3,4,2) pour
 * le premier joueur est 1er, le second est 2e, etc).
 * On suppose qu'il y a correpondance entre les indices de $S et les indices de $R.
 * @param array $S : scores actuels des joueurs
 * @param array $R : classement des joueurs
 * @param int $K : la quantité max de points gagnable ou perdable sur une partie
 * @return array : le différentiel de score de chaque joueur
 * @throws DomainException
 */
function ELO_classement($S, $R, $K)
{
    $m = count($S);
    if (count($R) != $m)
        throw new DomainException;

    $count = array_fill(0, $m, 0);
    $res = array_fill(0, $m, 0);
    for ($i = 0; $i < $m; $i++) {
        for ($j = $i + 1; $j < $m; $j++) {
            if ($R[$i] < $R[$j])
                $newSAB = ELO_duel($S[$i], 1, $S[$j], 0, $K);
            else
                $newSAB = ELO_duel($S[$i], 0, $S[$j], 1, $K);
            $res[$i] = 1 / ($count[$i] + 1) * ($count[$i] * $res[$i] + $newSAB[0]);
            $count[$i]++;
            $res[$j] = 1 / ($count[$j] + 1) * ($count[$j] * $res[$j] + $newSAB[1]);
            $count[$j]++;

        }
    }
    for ($i = 0; $i < $m; $i++) {
        if ($res[$i] > 0)
            $res[$i] = ceil($res[$i]);
        else
            $res[$i] = floor($res[$i]);
    }

    $e = array_sum($res);

    if($e != 0){
        $se = 0;

        if ($e < 0) {
            $se = min($res);
        }
        else if($e>0) {
            $se = max($res);
        }
        $i = array_search($se, $res);
        $res[$i] += -$e;
    }

    return $res;
}


/** Fonction qui calcule les différentiels de scores obtenus après une à count($S) joueurs
 * en considérant que $R correspond au score des joueurs à cette partie.
 * Ne prend pas en compte les écarts de scores entre les joueurs, la fonction ne construisant qu'un classement
 * entre les joueurs.
 * On suppose qu'il y a correpondance entre les indices de $S et les indices de $R.
 * @param $S
 * @param $R
 * @param $K
 * @param array $S : scores actuels des joueurs
 * @param array $R : scores des joueurs à une partie
 * @param int $K : la quantité max de points gagnable ou perdable sur une partie
 * @return array : le différentiel de score de chaque joueur
 * @throws DomainException
 */
function ELO_scores($S, $R, $K){
    $m = count($S);
    if (count($R) != $m)
        throw new DomainException;

    $R_classement = array_fill(0, $m, 0);
    arsort($R);
    $i = 1;
    foreach ($R as $k => $val){
        $R_classement[$k] = $i;
        $i++;
    }

    return ELO_classement($S, $R_classement, $K);
}

function random_classement($n)
{
    $r = array();
    for ($j=0; $j<$n; $j++)
    {
        array_push($r, $j+1);
    }
    shuffle($r);
    return $r;
}

/* Test */
function testeur(){
    print("<html><body><table><th>Itération</th><th>Joueur 1</th><th>Joueur 2</th><th>Joueur 3</th><th>Joueur 4</th><th>Joueur 5</th><th>Somme</th></tr>");
    $DS = array(1000,1000,1000,1000,1000);

    print("<tr><td>0</td>");
    foreach ($DS as $value) {
        print("<td>$value</td>");
    }
    $p = array_sum($DS);
    print("<td>$p</td></tr>");

    for($j=1;$j<1000;$j++){
        $sa = random_classement(5);
        $res = ELO_classement($DS, $sa, 32);
        print("<tr><td>$j</td>");

        for ($i=0;$i<count($DS);$i++) {
            $DS[$i] = $DS[$i] + $res[$i];
            $k1 = $DS[$i];
            $k2 = $res[$i];
            $k3 = $sa[$i];
            print("<td>$k1 ($k2) ($k3)</td>");
        }
        $p = array_sum($DS);
        print("<td>$p</td></tr>");
    }
    print("</table></body></html>");
}
